@@ -24,7 +24,7 @@ _abs_path = os.path.split(
 rcParams = dict(
         _path = _abs_path,
         filename                   = 'lingpy-'+str(date.today()),
-        timestamp                  = str(datetime.today()),
+        timestamp                  = datetime.strftime(datetime.today(),'%Y-%m-%d %H:%M'),
         answer_yes = ['y','Y','j','J','yes'],
         verbose = False,
         debug = False
