@@ -10,6 +10,7 @@ __author__="Johann-Mattis List"
 __date__="2013-03-07"
 
 
-from .multiple import *
-from .pairwise import *
-from .sca import *
+from .multiple import Multiple, mult_align
+from .pairwise import Pairwise, pw_align, nw_align, sw_align, we_align, \
+        structalign, turchin, edit_dist
+from .sca import MSA, PSA, Alignments, SCA
