@@ -1,7 +1,7 @@
 # author   : Johann-Mattis List, Johannes Dellert
 # email    : mattis.list@uni-marburg.de
 # created  : 2013-03-07 20:07
-# modified : 2013-11-23 09:11
+# modified : 2014-02-18 10:49
 
 """
 Basic module for pairwise and multiple sequence comparison.
@@ -14,7 +14,7 @@ perspective deals with aligned sequences.
 """
 
 __author__="Johann-Mattis List, Johannes Dellert"
-__date__="2013-11-23"
+__date__="2014-02-18"
 
 import numpy as np
 import re
@@ -33,7 +33,7 @@ from ..sequence.sound_classes import ipa2tokens, tokens2class, class2tokens, \
 from .multiple import Multiple
 from .pairwise import Pairwise
 from ..algorithm import misc
-
+from ._align import confidence
 
 class MSA(Multiple):
     """
@@ -175,7 +175,7 @@ class MSA(Multiple):
 
         # redefine the sequences of the Multiple class
         class_strings = [tokens2class(
-            seq.split('.'),
+            seq.split(' '),
             self.model,
             stress=keywords['stress']) for seq in self.seqs]
         
@@ -227,7 +227,8 @@ class MSA(Multiple):
 
         """
         defaults = dict(
-                wordlist = False
+                wordlist = False,
+                timestamp = False
                 )
         for k in defaults:
             if k not in keywords:
@@ -365,15 +366,17 @@ class MSA(Multiple):
             except:
                 pass
         
-        if fileformat not in ['html','tex']:
+        if fileformat not in ['html','tex'] and keywords['timestamp']:
             try:
-                out.write('# Created using LingPy-2.0\n')
+                out.write('# Created using LingPy-2.2\n')
                 if hasattr(self,'params'):
                     out.write('# Parameters: '+self.params+'\n')
                 out.write('# Created: {0}\n'.format(rcParams['timestamp']))
             except:
                 pass
             out.close()
+        elif fileformat not in ['html', 'tex']:
+            out.close()
 
 class PSA(Pairwise):
     """
@@ -777,9 +780,9 @@ class Alignments(Wordlist):
                     d['ID'] = []
                     if 'concept' in self.header:
                         concept = self[seqids[0],'concept']
-                        d['seq_id'] = 'Cognate Set: {0} ("{1}")'.format(key,concept)
+                        d['seq_id'] = '{0} ("{1}")'.format(key,concept)
                     else:
-                        d['seq_id'] = 'Cognate Set: {0}'.format(key)
+                        d['seq_id'] = '{0}'.format(key)
                     
                     # set up the data
                     for seq in seqids:
@@ -897,6 +900,23 @@ class Alignments(Wordlist):
         kw.update(keywords)
         if kw['defaults']: return kw
 
+        if str(kw['model']) == kw['model']:
+            kw['model'] = rcParams[kw['model']]
+
+        # create a params attribute
+        params = '_'.join(
+                [
+                    kw['method'],
+                    kw['model'].name,
+                    str(kw['gop']),
+                    '{0:.1f}'.format(kw['scale']),
+                    '{0:.1f}'.format(kw['factor']),
+                    kw['tree_calc'],
+                    '{0:.1f}'.format(kw['gap_weight']),
+                    kw['restricted_chars']
+                    ]
+                )
+
         # define a score-bar that shows how far the work is processed
         if not rcParams['verbose'] and rcParams['_sverb']:
             task_len = len(self.msa[kw['ref']])
@@ -947,7 +967,7 @@ class Alignments(Wordlist):
                         **kw
                         )
 
-                kw['sonar'] = sonars
+                kw['sonars'] = sonars
                 kw['classes'] = False
             
             if kw['method'] == 'progressive':
@@ -973,6 +993,13 @@ class Alignments(Wordlist):
 
             self._meta['msa'][kw['ref']][key]['alignment'] = m.alm_matrix
             self._meta['msa'][kw['ref']][key]['_sonority_consensus'] = m._sonority_consensus
+            self._meta['msa'][kw['ref']][key]['stamp'] = rcParams['align_stamp'].format(
+                    m.dataset,
+                    m.seq_id,
+                    rcParams['timestamp'],
+                    params
+                    )
+
 
             if kw['output']:
                 if kw['style'] in ['plain', 'msa']:
@@ -1034,7 +1061,29 @@ class Alignments(Wordlist):
             sys.stdout.write('|\r'+rcParams['_sverb_tbar_len'] * ' '+'     \r')
             sys.stdout.flush()
 
-                    
+    def get_confidence(self, scorer, ref="lexstatid", gap_weight=0.25):
+        """
+        Function creates confidence scores for a given set of alignments.
+
+        Parameters
+        ----------
+        scorer : :py:class:`~lingpy.algorithm._misc.ScoreDict`
+            A *ScoreDict* object which gives similarity scores for all segments in
+            the alignment.
+        ref : str (default="lexstatid")
+            The reference entry-type, referring to the cognate-set to be used for
+            the analysis.
+        gap_weight : {loat} (default=1.0)
+            Determine the weight assigned to matches containing gaps.
+
+        """
+        
+        corrs = confidence.get_confidence(self, scorer, ref, gap_weight)
+        if rcParams['verbose']:
+            print("[i] Successfully calculated confidence values for alignments.")
+        
+        return corrs
+
     def __len__(self):
         return len(self.msa)
 
@@ -1052,13 +1101,15 @@ class Alignments(Wordlist):
                 dataset = self.filename,
                 show = False,
                 filename = self.filename,
-                ref = rcParams['ref']
+                ref = rcParams['ref'],
+                confidence = False
                 )
         for k in defaults:
             if k not in keywords:
                 keywords[k] = defaults[k]
 
-        self.output('alm',ref=keywords['ref'],filename='.tmp')
+        self.output('alm',ref=keywords['ref'],filename='.tmp',
+                confidence=keywords['confidence'])
         html.alm2html(
                 '.tmp.alm',
                 **keywords
@@ -1278,6 +1329,7 @@ class Alignments(Wordlist):
                 filename = rcParams['filename'],
                 style = "id",
                 defaults = False,
+                confidence = False
                 )
         kw.update(keywords)
         if kw['defaults']: return kw
@@ -1332,15 +1384,33 @@ class Alignments(Wordlist):
                             cid = concept2id[concept]
                             # add this line for alignments containing loans
                             real_cogid = self[self.msa[ref][cogid]['ID'][i],ref]
+                            if not kw['confidence']:
+                                alm_string = '\t'.join(alm)
+                            else:
+                                confs = ['{0}'.format(x) for x in
+                                        self.msa[ref][cogid]['confidence'][i]]
+                                chars = [x for x in
+                                        self.msa[ref][cogid]['_charmat'][i]]
+                                alm_string = '\t'.join(
+                                        [a+'/'+b+'/'+c for a,b,c in zip(
+                                            alm,
+                                            confs,
+                                            chars
+                                            )
+                                            ]
+                                        )
+
                             out += '\t'.join(
                                 [
                                     str(real_cogid),
                                     taxon,
                                     concept,
                                     str(cid),
-                                    '\t'.join(alm)
+                                    alm_string, #'\t'.join(alm)
                                     ]
                                 )+'\n'
+
+
                     else:
                         this_idx = [x for x in self.etd[ref][cogid] if x != 0][0][0]
                         taxon = self[this_idx,'taxon']
@@ -1705,9 +1775,20 @@ def get_consensus(
                     tmp.items(),
                     key=lambda x:(x[1],len(x[0])),
                     reverse=True
-                    )] 
+                    )]
 
-                cons += [chars[0]]
+                # apply check for gaps here, if there are more gaps than in the
+                # full column, take the gaps, otherwise, take the next char
+                if chars[0] == '-':
+                    if tmp['-'] > sum([tmp[x] for x in tmp if x != '-']):
+                        cchar = '-'
+                    else:
+                        cchar = chars[1]
+                else:
+                    cchar = chars[0]
+                    
+                cons += [cchar]
+                #cons += [chars[0]]
 
     # otherwise, we use a bottom-up parsimony approach to determine the best
     # match
