@@ -1,5 +1,5 @@
 """
-Class for reading and tokenizing tabular data exported from a spreadsheet.
+Class for reading and tokenizing tabular data. e.g. csv data exported from a spreadsheet.
 """
 
 __author__="Steven Moran"
@@ -26,33 +26,61 @@ from ..convert import *
 
 class Spreadsheet:
     """
-    Basic class for reading tabular data exported from a spreadsheet.
+    Basic class to read tabular data (e.g. csv exported from spreadsheet), apply blacklist and pass to LingPy Wordlist.
 
-    This module provides a basic class for reading in a simple spreadsheet (delimited text file) for concepts and words in a set of languages.
+    This module provides a basic class for reading in a simple spreadsheet (delimited text file) 
+    for concepts and words in a set of languages. An optional blacklist can be used to remove / replace 
+    characters, items, words, etc., from the input data. Some simple analyses of 
 
     Parameters
     ----------
 
-    Notes
-    -----
-    TODO: how about a "config file" variable where the parameters below can be specified?
+    filename : str
+        Name of the input file.
+
+    fileformat : {None str}
+        If not specified the file <filename> will be loaded. Otherwise, the
+        fileformat is interpreted as the specific extension of the input file.
+
+    dtype : {list}
+        If not specified, all data will be loaded as strings. Otherwise, a
+        list specifying the data for each line should be provided.
+
+    comment : string (default="#")
+        Comment character in the begin of a line forces this line to be
+        ignored.
+
+    sep : string (default = "\t")
+        Specify the separator for the CSV-file.
+
+    language_id : string (default = "NAME")
+        A qualifier for columns in the input that contain language data.
+
+    meanings : string (default = "CONCEPT")
+        A qualifier for the concepts column in the input data.
+
+    blacklist : string (default = "")
+        Path and filename to the blacklist file.
+
+    cellsep : string (default = ";")
+        The character(s) indiciating the separtor between multiple linguistic forms in the same cell.
+
+    skip_empty_cells : boolean (default = True)
+        Flag for skipping empty cells in the input.
+
 
     """
     def __init__(self, 
                  filename,
-                 fileformat = None, # req'd in read.csv
-                 dtype = None, # req' in read.csv 
+                 fileformat = None, # required in lingpy/read/csv.py
+                 dtype = None, # required in lingpy/read/csv.py
                  comment = '#', # comment marker
                  sep = '\t', # column separator
                  language_id = "NAME", # language column identifier
-                 # lang_cols = [], # int specified columns for languages in a spreadsheet (index starts at 0)
                  meanings = "CONCEPT", # explicit name of column containing concepts
                  blacklist = "", # filename path to blacklist file
-                 conf = "", # spreadsheet .rc file
-                 cellsep = ';', # cell separator, separates forms in the same cell; delimited with pipe "|"
-                 verbose = False,
+                 cellsep = ';',
                  skip_empty_concepts = True,
-                 profiles = {}, # file path and names to orthography profiles
                  **keywords
                  ):
 
@@ -62,27 +90,22 @@ class Spreadsheet:
         self.comment = comment
         self.sep = sep
         self.language_id = language_id
-        # self.lang_cols = lang_cols
         self.meanings = meanings
         self.blacklist = blacklist
-        self.conf = conf
         self.cellsep = cellsep
         self.skip_empty_concepts = skip_empty_concepts
-        self.profiles = profiles
-        self.verbose = verbose
 
-        # set up matrix
+        # set up matrix and normalize cell contents
         self.matrix = []
         self._init_matrix()
         self._normalize()
 
+        # run blacklist replacements
         if self.blacklist: self._blacklist()
 
-        if self.profiles: 
-            self.tokenized_matrix = self._tokenize(profiles)
-
+        # prepare dictionary for automatic pass-off to LingPy Wordlist
         self._prepare()
-        # self._prepare_tokenized()
+
 
     def _init_matrix(self):
         """
@@ -132,52 +155,30 @@ class Spreadsheet:
                     matrix_row.append(spreadsheet[i][j])
             self.matrix.append(matrix_row)
 
-        """
-        matrix_header = []
-        matrix_header.append(header[self.concepts])        
-        for i in language_indices:
-            matrix_header.append(header[i].replace(self.language_id, "").strip())
-        self.matrix.append(matrix_header)
-
-        # append the concepts and words in languages and append the rows
-        for i in range(1, len(spreadsheet)): # skip the header row
-            matrix_row = [] # collect concepts and languages to add to matrix
-            temp = []
-            for j in range(0, len(spreadsheet[i])):
-                if j == self.concepts:
-                    matrix_row.append(spreadsheet[i][j])
-                if j in language_indices:
-                    temp.append(spreadsheet[i][j])
-            for item in temp:
-                print(item)
-                matrix_row.append(item)
-            self.matrix.append(matrix_row)
-            """
-
     def _normalize(self):
         """ 
-        Function to Unicode normalize (NFD) cells in the matrix.
+        Function to Unicode normalize (NFD) cells in the input matrix.
         """
         for i in range(0, len(self.matrix)):
             for j in range(0, len(self.matrix[i])):
                 normalized_cell = unicodedata.normalize("NFD", self.matrix[i][j])
                 if not normalized_cell == self.matrix[i][j]:
-                    if self.verbose:
+                    if rcParams['debug']:
                         print("[i] Cell at <"+self.matrix[i][j]+"> ["+str(i)+","+str(j)+"] not in Unicode NFD. Normalizing.")
                     self.matrix[i][j] = normalized_cell
     
-    def _prepare(self,full_rows = False):
+    def _prepare(self, full_rows = False):
         """
         Prepare the spreadsheet for automatic pass-on to Wordlist.
+
+        Notes
+        -----
+        We now assume that the matrix only contains concepts and counterparts.
+
         """
-        # XXX we now assume that the matrix is 'normalized',i.e. that it only
-        # contains concepts and counterparts, in later versions, we should make
-        # this more flexible by adding, for example, also proto-forms, or
-        # cognate ids
 
         # define a temporary matrix with full rows
         if not full_rows:
-            # matrix = self.matrix
             matrix = self.matrix
         else:
             matrix = self.get_full_rows()
@@ -192,18 +193,13 @@ class Spreadsheet:
             if line:
                 # get the concept
                 concept = line[0].strip()
-
                 if concept:
-
                     # get the rest
                     for j,cell in enumerate(line[1:]):
-
                         # get the language
                         language = matrix[0][j+1].replace(self.language_id,'').strip()
-
                         # get the counterparts
                         counterparts = [x.strip() for x in cell.split(self.cellsep)]
-
                         # append stuff to dictionary
                         for counterpart in counterparts:
                             if counterpart:
@@ -224,24 +220,17 @@ class Spreadsheet:
         # make a simple header for wordlist import
         self.header = dict([(a,b) for a,b in zip(d[0],range(len(d[0])))])
 
-
-
-
     def _blacklist(self):
         """
         Remove anything in the spreadsheet that's specified in the blacklist file.
 
-        Notes
-        -----
-
-        TODO: make parameter **kwargs
         """
         if not os.path.isfile(self.blacklist):
             if rcParams['verbose']:
                 print("[i] There is no blacklist specified at the follow file path location. Proceeding without blacklist.")
             return
 
-        blacklist_file = codecs.open(self.blacklist, "r",'utf-8')
+        blacklist_file = codecs.open(self.blacklist, "r", "utf-8")
         # loop through the blacklist file and compile the regexes
         rules = []
         replacements = []
@@ -271,8 +260,8 @@ class Spreadsheet:
                     match = rules[k].search(self.matrix[i][j])
                     if not match == None:
                         match = re.sub(rules[k], replacements[k], self.matrix[i][j])                
-                        if self.verbose:
-                            print("[i] Replacing <"+self.matrix[i][j]+"> ["+str(i)+","+str(j)+"] with <"+match+">.")
+                        if rcParams['debug']:                    
+                            print("[i] Replacing ["+str(i)+","+str(j)+"] <"+self.matrix[i][j]+"> with <"+match+">.")
                         self.matrix[i][j] = match.strip()
 
     def pprint(self, matrix, delimit="\t"):
@@ -329,8 +318,8 @@ class Spreadsheet:
                 cell = self.matrix[i][j].strip()
                 if cell == "":
                     # TODO: integrate global verbosity
-                    # if self.verbose:
-                        # print("[i] Missing cell")
+                    if rcParams['verbose']:
+                        print("[i] Missing cell")
                     continue
                 # unicode characters
                 for char in cell:
@@ -387,149 +376,18 @@ class Spreadsheet:
             type_matrix.append(result)
         return type_matrix
 
-
-    def _tokenize(self, profiles):
-        """ 
-        Takes a dictionary or list of kwargs that specifies "column name" and "path to orthography profile".
-        This function lets a user specify different orthographic profiles per column in a spreadsheet that's 
-        already been semantically aligned.
-        """
-        tokenized_matrix = copy.deepcopy(self.matrix)
-
-        # get orthography parsers and store them in dict
-        ops = {}
-        for k, v in profiles.items():
-            ops[k] = Tokenizer(v) #, debug=True)
-
-        # TODO: identify the column and orthographically parse it in the matrix
-
-        # apply tokenization
-        header = tokenized_matrix[0]
-        for i in range(1, len(tokenized_matrix)):
-            for j in range(0, len(tokenized_matrix[i])):
-                cell = tokenized_matrix[i][j].strip()
-                if cell == "":
-                    continue
-
-                # split up tokens by the cell separtor and grapheme-ically parse them
-                tokens = cell.split(self.cellsep)
-                tokenized_tokens = []
-                for token in tokens:
-                    token = token.strip()
-                    if header[j] in ops:
-                        tokenized_form = ops[header[j]].transform_rules(token).strip() # .replace("#", "").strip()
-                        if tokenized_form == "":
-                            tokenized_tokens.append("FAIL")
-                        else:
-                            tokenized_tokens.append(tokenized_form)
-                            tokenized_matrix[i][j] = " \\\\ ".join(tokenized_tokens)
-
-                    # original
-                    # tokenized_matrix[i][j] = ops[header[j]].parse_graphemes(cell).strip() # .replace("#", "").strip()
-                    # print(ops[header[j]].parse_graphemes(cell).strip()) # .replace("#", "").strip()
-                    else:
-                        if header[j] in ops:
-                            tokenized_form = ops[header[j]].transform_rules(cell).strip() # .replace("#", "").strip()
-                            if tokenized_form == "":
-                                tokenized_matrix[i][j] = "FAIL"
-                            else:
-                                tokenized_matrix[i][j] = tokenized_form                    
-        return tokenized_matrix
-
-    def _prepare_tokenized(self, full_rows=False):
-        """
-        # Prepare the spreadsheet(s) for automatic pass-on to Wordlist.
-        # We now assume that the matrix contains only concepts and counterparts, 
-        # TODO: in later versions, we should make this more flexible by adding, 
-        # for example, also proto-forms, or cognate ids, etc.
-        """
-
-        """
-        # define temp matrices with full rows if specified
-        if not full_rows:
-            matrix = self.matrix
-            tokenized_matrix = self.tokenized_matrix
-        else:
-            matrix = self.get_full_rows(self.matrix)
-            tokenized_matrix = self.get_full_rows(self.tokenized_matrix)
-            """
-        matrix = self.matrix
-        tokenized_matrix = self.tokenized_matrix
-
-        # store and process the data
-        wordlist = {}
-        id = 1
-        for i in range(1, len(tokenized_matrix)):
-            # get concept
-            if tokenized_matrix[i]:
-                concept = tokenized_matrix[i][0].strip()
-                # proceed only if there is a concept
-                if concept:
-                    for j in range(1, len(tokenized_matrix[i])):
-                        language = tokenized_matrix[0][j].replace(self.language_id, "").strip()
-                        counterparts = [x.strip() for x in tokenized_matrix[i][j].split(self.cellsep)]
-                        # if a tokenized matrix, gather the tokenized forms
-                        tokenized_counterparts = []
-                        if self.tokenized_matrix:
-                            tokenized_counterparts = [y.strip() for y in tokenized_matrix[i][j].split(self.cellsep)]
-
-                        if not len(counterparts) == len(tokenized_counterparts):
-                            print(concept)
-                            print(self.matrix[i])
-                            print(self.tokenized_matrix[i])
-                            print(self.matrix[i][j])
-                            print(len(counterparts), counterparts)
-                            print(self.tokenized_matrix[i][j])
-                            print(len(tokenized_counterparts), tokenized_counterparts)
-                            print()
-
-
-                        # append stuff to dictionary
-                        for k in range (0, len(counterparts)):
-                            if self.tokenized_matrix:
-                                if counterparts[k]:
-                                    # wordlist[id] = [concept, language, counterparts[k], tokenized_counterparts[k]]
-                                    wordlist[id] = [concept, language, counterparts[k]]
-                                    id += 1
-                            else:
-                                if counterparts[k]:
-                                    wordlist[id] = [concept, language, counterparts[k]]
-                                    id += 1
-
-        # add the header to the wordlist dictionaries
-        if self.tokenized_matrix:
-            wordlist[0] = ["concept", "doculect", "counterpart", "tokens"]
-        else:
-            wordlist[0] = ["concept", "doculect", "counterpart"]
-
-        # make the dictionary an attribute of this spreadsheet object
-        self._data = dict([(k, v) for k, v in wordlist.items() if k > 0])
-
-        # make empty meta-attribute
-        self._meta = dict(
-                filename = self.filename
-                )
-
-        # make a simple header for wordlist import
-        self.header = dict([(a, b) for a, b in zip(wordlist[0], range(len(wordlist[0])))])
-
-
     def get_full_rows(self, matrix):
         """
         Create a 2D matrix from only the full rows in the spreadsheet.
         """
         full_row_matrix = []
-
         for row in matrix:
             is_full = 1
-
             for token in row:
                 if token == "":
                     is_full = 0
-
             if is_full:
                 full_row_matrix.append(row)
-
         return(full_row_matrix)
 
     def print_doculect_character_counts(self, doculects=1):
@@ -603,7 +461,6 @@ class Spreadsheet:
         """
         Output the matrix into Harry Potter format.
         """
-
         defaults = dict(
                 filename = "lingpy-{0}".format(_timestamp()),
                 meta = self._meta
@@ -632,5 +489,3 @@ class Spreadsheet:
         """
 
         return self._output(fileformat,**keywords)
-
-
