@@ -1,13 +1,13 @@
 # author   : Johann-Mattis List
 # email    : mattis.list@gmail.com
 # created  : 2013-03-14 00:21
-# modified : 2013-11-13 10:42
+# modified : 2014-02-13 12:14
 """
 This module provides a basic class for the handling of word lists.
 """
 
 __author__="Johann-Mattis List"
-__date__="2013-11-13"
+__date__="2014-02-13"
 
 import os
 import sys
@@ -473,23 +473,23 @@ class Wordlist(QLCParser):
 
         Note that this function returns 0 for missing values (concepts that
         don't have a word in the given language). If one wants to avoid this,
-        the 'flat' keyword should be set to c{True}.
+        the 'flat' keyword should be set to *True*.
         
         Select all words (orthographical representation) for the concept
         "Harry"::
         
             >>> wl.get_list(concept="Harry",entry="words")
-            [['hæri', 'haralt', 'gari', 'gari']]            
+            [['Harry', 'Harald', 'Гари', 'Гарi']]            
         
         Note that the values of the list that is returned are always
         two-dimensional
         lists, since it is possible that the original file contains synonyms
         (multiple words corresponding to the same concept). If one wants to
         have a flat representation of the entries, the 'flat' keyword should be
-        set to c{True}::
+        set to *True*::
 
             >>> wl.get_list(concept="Harry",entry="words",flat=True)
-            ['hæri', 'haralt', 'gari', 'gari']
+            ['hæri', 'haralt', 'gari', 'hari']
             
         See also
         --------
@@ -966,7 +966,6 @@ class Wordlist(QLCParser):
                     entry=keywords['entry'],
                     missing=keywords['missing']
                     )
-            print('ok')
             if fileformat == 'paps.nex':
                 pap2nex(
                         self.taxa,
@@ -1048,16 +1047,21 @@ class Wordlist(QLCParser):
                             idx = self._header[key]
                             stmts += ["line[{0}] ".format(idx)+value]
 
+                if rcParams['debug']: print("calculated what should be excluded")
+
                 # get the data
                 out = {}
 
                 for key,line in self._data.items():
+                    if rcParams['debug']: print(key)
+                    
                     if rows:
                         if eval(" and ".join(stmts)):
                             out[key] = [line[i] for i in indices]
                     else:
                         out[key] = [line[i] for i in indices]
-
+                
+                if rcParams['debug']: print("passing data to wl2qlc")
                 wl2qlc(
                         header,
                         out,
@@ -1430,6 +1434,6 @@ class Wordlist(QLCParser):
             Tokenization target.
 
         """
-        self._tokenize(orthography_profile, source, target, column,
-            **keywords)
+        self._tokenize(orthography_profile=orthography_profile, source=source, target=target,
+                column=column, **keywords)
 
