@@ -1,13 +1,13 @@
 # author   : Johann-Mattis List
 # email    : mattis.list@gmail.com
 # created  : 2013-03-12 11:56
-# modified : 2013-11-24 23:09
+# modified : 2014-04-28 21:04
 """
 LexStat algorithm for automatic cognate detection.
 """
 
 __author__="Johann-Mattis List"
-__date__="2013-11-24"
+__date__="2014-04-28"
 
 # builtin
 import random
@@ -32,6 +32,7 @@ from ..read.phylip import read_scorer # for easy reading of scoring functions
 from ..algorithm import clustering
 
 from ..algorithm import calign
+from ..algorithm import talign
 from ..algorithm import misc
 
 class LexStat(Wordlist):
@@ -307,7 +308,7 @@ class LexStat(Wordlist):
                     if i < j:
                         
                         # add dictionary scores to the scoredict
-                        score = kw["model"](
+                        score = self.model(
                                 charA[charA.index('.')+1][0],
                                 charB[charB.index('.')+1][0]
                                 )
@@ -315,7 +316,7 @@ class LexStat(Wordlist):
                         matrix[j][i] = score
                     elif i == j:
                         # add dictionary scores to the scoredict
-                        score = kw["model"](
+                        score = self.model(
                                 charA[charA.index('.')+1][0],
                                 charB[charB.index('.')+1][0]
                                 )
@@ -335,7 +336,7 @@ class LexStat(Wordlist):
                     if i < j:
                         
                         # add dictionary scores to the scoredict
-                        score = kw["model"](
+                        score = self.model(
                                 charA[0],
                                 charB[0]
                                 )
@@ -343,7 +344,7 @@ class LexStat(Wordlist):
                         matrix[j][i] = score
                     elif i == j:
                         # add dictionary scores to the scoredict
-                        score = kw["model"](
+                        score = self.model(
                                 charA[0],
                                 charB[0]
                                 )
@@ -1276,7 +1277,8 @@ class LexStat(Wordlist):
         """
         # currently, there are no defaults XXX
         kw = dict(
-                defaults = False
+                defaults = False,
+                external_scorer = False, # external scoring function
                 )
         kw.update(keywords)
 
@@ -1345,6 +1347,17 @@ class LexStat(Wordlist):
                     self[idxB,'tokens']
                     )
 
+        elif method == 'custom':
+            
+            function = lambda idxA,idxB: talign.align_pair(
+                    self[idxA, 'utokens'],
+                    self[idxB, 'utokens'],
+                    gop,
+                    scale,
+                    keywords['external_scorer'],
+                    'overlap',
+                    True)[2]
+
         if not concept:
             concepts = sorted(self.rows)
         else:
@@ -1446,6 +1459,7 @@ class LexStat(Wordlist):
                 link_threshold  = False,
                 _return_matrix  = False, # help function for test purposes
                 defaults        = False,
+                external_scorer = False, # external scoring dictionary
                 )
         kw.update(keywords)
         if kw['defaults']: return kw
@@ -1464,7 +1478,7 @@ class LexStat(Wordlist):
                 )
         self._stamp += '# Cluster: ' + self.params['cluster']
         
-        if method not in ['lexstat','sca','turchin','edit-dist']:
+        if method not in ['lexstat','sca','turchin','edit-dist', 'custom']:
             raise ValueError(
                     "[!] The method you selected is not available."
                     )
@@ -1551,6 +1565,21 @@ class LexStat(Wordlist):
                     )
                 thresholds += [t]
             threshold = sum(thresholds) / len(thresholds)
+        # new method for threshold estimation based on calculating approximate
+        # random distributions of similarities for each sequence
+        elif kw['guess_threshold'] and kw['gt_mode'] == 'nulld':
+            DR = []
+            align = lambda x,y: self.align_pairs(x, y, method=method,
+                    restricted_chars=restricted_chars, mode=mode, scale=scale,
+                    factor=factor, return_distance=True, pprint=False, gop=gop)
+            for l1,l2 in self.pairs:
+                if l1 != l2:
+                    pairs = self.pairs[l1,l2]
+                    for p1,p2 in pairs:
+                        dx = [align(p1, pairs[random.randint(0, len(pairs)-1)][1])
+                                for i in range(len(pairs)//5)]
+                        DR += dx #[sum(dx)/len(dx)]
+            threshold = sum(DR) / len(DR)
         
         # iterate over the matrices
         for concept,indices,matrix in matrices:
@@ -1567,6 +1596,10 @@ class LexStat(Wordlist):
                         matrix,
                         kw['gt_trange']
                         )
+            # considering new function here JML
+            elif kw['guess_threshold'] and kw['gt_mode'] == 'nullditem':
+                for idx in indices:
+                    pass
             else:
                 t = threshold
 
@@ -1617,6 +1650,8 @@ class LexStat(Wordlist):
                 self.add_entries('lexstatid',clr,lambda x:x,override=override)
             elif method == 'sca':
                 self.add_entries('scaid',clr,lambda x:x,override=override)
+            elif method == 'custom':
+                self.add_entries('customid',clr, lambda x:x, override=override)
             else:
                 self.add_entries('editid',clr,lambda x:x,override=override)       
         else:
@@ -1719,10 +1754,11 @@ class LexStat(Wordlist):
             gop = -2,
             scale = 0.5,
             factor = 0.3,
-            restricted_chars = 'T_'
+            restricted_chars = 'T_',
+            aggregate = True
             ):
         """
-        Method calculates randoms scores for unrelated words in a dataset.
+        Method calculates different distance estimates for language pairs.
 
         Parameters
         ----------
@@ -1741,6 +1777,9 @@ class LexStat(Wordlist):
         restricted_chars : str (default="T_")
             Select the restricted chars (boundary markers) in the prosodic
             strings in order to enable secondary alignment.
+        aggregate : bool (default=True)
+            Return aggregated distances in form of a distance matrix for all
+            taxa in the data.
 
         Returns
         -------
@@ -1765,22 +1804,40 @@ class LexStat(Wordlist):
         else:
             function = lambda x,y: edit_dist(
                     self[x,'tokens'],
-                    self[y,'tokens']
+                    self[y,'tokens'],
+                    normalized=True
                     )
+        if not aggregate:
+            for i,taxA in enumerate(self.taxa):
+                for j,taxB in enumerate(self.taxa):
+                    if i < j:
 
-        for i,taxA in enumerate(self.taxa):
-            for j,taxB in enumerate(self.taxa):
-                if i < j:
+                        # get a random selection of words from both taxa
+                        sample_pairs = self.pairs[taxA,taxB]
+                        
+                        for pA,pB in sample_pairs:
+                            d = function(pA,pB)
 
-                    # get a random selection of words from both taxa
-                    sample_pairs = self.pairs[taxA,taxB]
-                    
-                    for pA,pB in sample_pairs:
-                        d = function(pA,pB)
+                            D += [d]
+            D = sorted(D)
+        else:
+            for i,taxA in enumerate(self.taxa):
+                for j,taxB in enumerate(self.taxa):
+                    if i < j:
+                        sample_pairs = self.pairs[taxA,taxB]
+                        
+                        distances = []
+                        for pA,pB in sample_pairs:
+                            try:
+                                d = function(pA,pB)
+                            except:
+                                print("Zero-Warning")
+                                d = 1.0
+                            distances += [d]
+                        D += [sum(distances) / len(distances)]
+            D = misc.squareform(D)
 
-                        D += [d]
-
-        return sorted(D)
+        return D
 
     def output(
             self,
