@@ -1,23 +1,25 @@
 # author   : Johann-Mattis List
 # email    : mattis.list@uni-marburg.de
 # created  : 2013-10-10 16:31
-# modified : 2013-11-24 18:31
+# modified : 2014-03-10 20:31
 """
 Basic functions for HTML-plots.
 """
 
 __author__="Johann-Mattis List"
-__date__="2013-11-24"
+__date__="2014-03-10"
 
 
 import os
 import colorsys
 import codecs
 import webbrowser
+import json
+import re
 
 from ..settings import rcParams
 from ..read.qlc import read_msa
-from ..sequence.sound_classes import pid
+from ..sequence.sound_classes import pid, token2class, tokens2class, ipa2tokens
 
 import numpy as np
 
@@ -61,6 +63,7 @@ def alm2html(
         main_template = '',
         table_template = '',
         dataset = '',
+        confidence = False,
         **keywords
         ):
     """
@@ -90,6 +93,14 @@ def alm2html(
     lingpy.convert.html.msa2tex
 
     """
+    defaults = dict(
+            json = "",
+            labels = {},
+            )
+    for k in defaults:
+        if k not in keywords:
+            keywords[k] = defaults[k]
+            
     # open the infile
     try:
         data = codecs.open(infile, "r", "utf-8").read()
@@ -120,17 +131,55 @@ def alm2html(
     if table_template:
         table = codecs.open(table_template,'r','utf-8').read()
     else:
-        table = codecs.open(
-                os.path.join(
-                    rcParams['_path'],
-                    'data',
-                    'templates',
-                    'alm2html.table.html'
-                    ),
-                'r',
-                'utf-8'
-                ).read()
-
+        if not confidence:
+            table = codecs.open(
+                    os.path.join(
+                        rcParams['_path'],
+                        'data',
+                        'templates',
+                        'alm2html.table.html'
+                        ),
+                    'r',
+                    'utf-8'
+                    ).read()
+        else:
+            table = codecs.open(
+                    os.path.join(
+                        rcParams['_path'],
+                        'data',
+                        'templates',
+                        'alm2html.table.js.html'
+                        ),
+                    'r',
+                    'utf-8'
+                    ).read()
+    
+    # load css
+    css = codecs.open(
+            os.path.join(
+                rcParams['_path'],
+                'data',
+                'templates',
+                'alm.css'
+                ),
+            'r',
+            'utf-8'
+            ).read()
+    
+    # load js
+    js = codecs.open(
+            os.path.join(
+                rcParams['_path'],
+                'data',
+                'templates',
+                'alm.js'
+                ),
+            'r',
+            'utf-8'
+            ).read()
+    
+    # define a label function for the taxa
+    label = lambda x: keywords['labels'][x] if x in keywords['labels'] else x
 
     # check for windows-compatibility
     data = data.replace(os.linesep,'\n')[:-1]
@@ -152,6 +201,7 @@ def alm2html(
         lines = block.split('\n')
         
         m = []
+        confs = []
         for l in lines:
             m.append(l.split('\t'))
         
@@ -174,7 +224,7 @@ def alm2html(
                     colors.append((i,'white'))
                     white = False
                 else:
-                    colors.append((i,'lightgray'))
+                    colors.append((i,'gray'))
                     white = True
             colors = dict(colors)
         
@@ -187,22 +237,27 @@ def alm2html(
                 NAME=iName,
                 ID = iID)
         # define the basic string for the insertion
-        bas = '<tr style="background-color:{0};font-weight:{2}">\n{1}\n</tr>'
+        bas = ' <tr class="{0}{2} taxon" taxon="{3}">\n{1}'
 
         for tracer,l in enumerate(m):
             # check whether the current line is a borrowing
             if int(l[0]) < 0:
-                loan_line = 'bold'
+                loan_line = ' loan'
             else:
-                loan_line = 'normal'
+                loan_line = ''
 
             # assign the cognate id
-            tmp = '<td>{0}</td>\n'.format(l[0])
-            tmp += '<td>{0}</td>\n'.format(l[1].strip('.'))
-            tmp += '<td>{0}</td>\n'.format(''.join(l[4:]).replace('-',''))
-            tmp += '<td style="background-color:{0}">'.format(colors[abs(int(l[0]))])
-            tmp += '<table style="background-color:{0}">\n'.format(colors[abs(int(l[0]))])
-            tmp += '<tr>\n{0}\n</tr>\n</table>\n'
+            tmp = '  <td>{0}</td>\n'.format(l[0])
+            tmp += '  <td>{0}</td>\n'.format(label(l[1].strip('.')))
+
+            # check alignments for confidence scores
+            ipa_string = ''.join([cell.split('/')[0] for cell in
+                l[4:]]).replace('-', '')
+
+            tmp += '  <td>{0}</td>\n'.format(ipa_string)
+            tmp += '  <td class="{0}">\n'.format(colors[abs(int(l[0]))])
+            tmp += '   <table class="{0}">\n'.format(colors[abs(int(l[0]))])
+            tmp += '    <tr>\n{0}    </tr>\n   </table>\n  </td>\n </tr>\n'
             
             # check whether another entry follows that is also an alignment,
             # otherwise, there's no need to display a word as an alignment
@@ -213,31 +268,62 @@ def alm2html(
             if tracer > 0:
                 if abs(int(m[tracer-1][0])) == abs(int(l[0])):
                     cognate_set = True
+            
+            # fill out html for the cognate sets
+            if cognate_set:
 
-            if cognate_set: #len(l[4:]) > 1:
                 alm = ''
                 for char in l[4:]:
-                    char = char
-                    error = False
-                    try:
-                        c = rcParams['_color'][char]
-                    except:
+                    
+                    # check for confidence scores
+                    if '/' in char:
                         try:
-                            c = rcParams['_color'][char[0]]
-                        except:
-                            c = 'white'
-                            error = True
-                    alm += '<td style="width:30px;text-align:center;'
-                    if error:
-                        alm += 'background-color:{0};color:red;font-weight:bold">{1}</td>'.format(c,char)
-                       
+                            char,conf,num = char.split('/')
+                            conf = int(conf)
+                        except ValueError:
+                            print(char.split('/'))
+                            raise ValueError("Something is wrong with %s." % (char))
+
                     else:
-                        alm += 'background-color:{0};color:white;font-weight:bold;">{1}</td>'.format(c,char)
+                        char,conf,rgb = char,(255,255,255),0.0
+                    
+                    if char == '-':
+                        d = 'dolgo_GAP'
+                        c = '#bbbbbb'
+                    else:
+                        d = 'dolgo_'+token2class(char, rcParams['dolgo'])
+                        c = token2class(char, rcParams['_color'])
+
+                        # bad check for three classes named differently
+                        if d == 'dolgo__':
+                            d = 'dolgo_X'
+                        elif d == 'dolgo_1':
+                            d = 'dolgo_TONE'
+                        elif d == 'dolgo_0':
+                            d = 'dolgo_ERROR'
+                    
+                    if confidence:
+                        alm += '     '
+                        alm += '<td class="char {1}" confidence={0} '.format(
+                                conf,
+                                d
+                                )
+                        alm += 'char="{0}" '.format(char)
+                        alm += 'onclick="'+"show('{0}')".format(num)+'" '
+                        alm += 'num="{0}"'.format(num)
+                        alm += '>\n      {0}\n     </td>\n'.format(char)
+                    else:
+                        alm += '     '
+                        alm += '<td class="char {0}">{1}</td>\n'.format(d,char)
             else:
-                alm = '<td style="border-color:{0};background-color:{1};">{0}'.format('--',colors[abs(int(l[0]))])
+                alm = '      '
+                alm += '<td class="{0}">--</td>\n'.format(colors[abs(int(l[0]))])
             
             # format the alignment
-            tmp = tmp.format(alm)
+            try:
+                tmp = tmp.format(alm)
+            except ValueError:
+                raise ValueError("Unknown problem in matchin %s and %s." % (alm, tmp))
 
             # check for last line, where a new line should be inserted (not the
             # fastest solution, but plotting is not a matter of time, and it
@@ -245,14 +331,18 @@ def alm2html(
             if tracer < len(m)-1:
                 pass
             else:
-                tmp += '<tr style="background-color:white;"><td colspan="4">'
-                tmp += '<hr style="background-color:white;border-color:white;border:0;height:2pt;"/>\n'
+                if confidence:
+                    tmp += ' </table>\n'
+
+                tmp += ' <tr class="empty"><td colspan="4" class="empty">'
+                tmp += '<hr class="empty" /></td></tr>\n'
             
             # format the whole string
             tmp_str += bas.format(
                     colors[abs(int(l[0]))],
                     tmp,
-                    loan_line
+                    loan_line,
+                    l[1]
                     )
 
     if not title:
@@ -260,11 +350,18 @@ def alm2html(
     if not shorttitle:
         shorttitle = "LexStat"
 
+    # check for json-attribute
+    if keywords['json']:
+        keywords['json'] = 'var myjson = '+json.dumps(keywords['json'],
+                indent=1)
+
     html = html.format(
             shorttitle = shorttitle,
             title = title,
             table = tmp_str,
             dataset = dataset,
+            javascript = js,
+            css = css,
             **keywords
             )
 
@@ -346,7 +443,12 @@ def msa2html(
     lingpy.convert.html.alm2html
     """
     defaults = dict(
-            pid_mode = 1
+            pid_mode = 1,
+            stress = rcParams['stress'],
+            css = False,
+            js = False,
+            compact = False,
+            class_sort = True
             )
     for k in defaults:
         if k not in keywords:
@@ -361,8 +463,21 @@ def msa2html(
     # load templates
     if not template:
         html = codecs.open(os.path.join(path,'msa2html.html'),'r','utf-8').read()
+    elif template == 'js':
+        html = codecs.open(os.path.join(path, 'msa2html.js.html'), 'r',
+                'utf-8').read()
     else:
         html = codecs.open(template,'r','utf-8').read()
+
+    if not keywords['css']:
+        css = codecs.open(os.path.join(path,'msa.css'), 'r', 'utf-8').read()
+    else:
+        css = codecs.open(keywords['css'], 'r', 'utf-8').read()
+
+    if not keywords['js']:
+        js = codecs.open(os.path.join(path,'msa.js'), 'r', 'utf-8').read()
+    else:
+        js = codecs.open(keywords['js'], 'r', 'utf-8').read()
     
     # treat the msa-object as a file and try to load the file if this is the
     # case
@@ -397,16 +512,16 @@ def msa2html(
     # determine the length of the longest taxon
     taxl = max([len(t) for t in msa['taxa']])
 
+    # format css file 
+    css = css.replace('TAXON_LENGTH', str(taxl * 10))
+
     out = ''
-    tr = '<tr class="msa">\n{0}\n</tr>'
-    td_taxon = '<td class="taxon" width="'+str(15 * taxl)+'">{0}</td>\n'
+    tr = '<tr class="msa" unique="{1}" taxon={2} sequence={3}>{0}</tr>\n'
+    td_taxon = '<td class="taxon">{0}</td>'
     perc = int(80 / len(msa['alignment'][0]) + 0.5)
-    td_residue = '<td class="residue" width="50" align="center" bgcolor="{1}">'+\
-            '{0}</td>\n'
-    td_swap = '<td class="residue swap" style="border:solid 3px black" width="50"'+\
-            'align="center" bgcolor="{1}">{0}</td>\n'
-    td_unaligned = '<td class="residue noalign" style="border:dotted 1px gray"'+\
-            'width="50" align="center" bgcolor="white">{0}</td>\n'
+    td_residue = '<td class="residue {1}">{0}</td>'
+    td_swap = '<td class="residue swap {1}">{0}</td>'
+    td_unaligned = '<td class="residue noalign {1}">{0}</td>'
     
     # check for swaps in the alignment
     if 'swaps' in msa:
@@ -422,26 +537,65 @@ def msa2html(
         local = ['.'] * len(msa['alignment'][0])
         for i in msa['local']:
             local[i] = '*'
+    
+    # get two sorting schemas for the sequences
+    if keywords['class_sort']:
+
+        classes = [tokens2class(ipa2tokens(seq), rcParams['asjp']) for seq in msa['seqs']]
+        seqs = dict(
+                [(a[1],b) for a,b in zip(
+                    sorted(
+                        zip(classes,msa['seqs']),
+                        key = lambda x: x[0] #list(zip(x[0],x[1]))
+                        ),
+                    range(1,len(msa['seqs'])+1)
+                    )]
+                )
+    else:
+        seqs = dict(zip(sorted(msa['seqs']), range(1,len(msa['seqs'])+1)))
+    taxa = dict(zip(sorted(msa['taxa']), range(1,len(msa['taxa'])+1)))
+
+    # set up a list to store unique alignments
+    alignments = []
 
     # start iteration
     for i,taxon in enumerate(msa['taxa']):
         tmp = ''
         tmp += td_taxon.format(taxon)
+        
+        # append alignment to alignments
+        alignment = ''.join(msa['alignment'][i])
+        sequence = msa['seqs'][i]
+        if alignment in alignments:
+            unique = 'false'
+        else:
+            unique = 'true'
+            alignments += [alignment]
+
         for j,char in enumerate(msa['alignment'][i]):
-            try:
-                c = rcParams['_color'][char]
-            except:
-                try:
-                    c = rcParams['_color'][char[0]]
-                except:
-                    print(char)
-            if j in swaps:
-                tmp += td_swap.format(char,c)
-            elif local[j] != '*':
-                tmp += td_unaligned.format(char,c)
+            if char == '-':
+                d = 'dolgo_GAP'
+                c = '#bbbbbb'
             else:
-                tmp += td_residue.format(char,c)
-        out += tr.format(tmp)
+                d = 'dolgo_'+token2class(char, rcParams['dolgo'])
+                c = token2class(char, rcParams['_color'])
+
+                # bad check for three classes named differently
+                if d == 'dolgo__':
+                    d = 'dolgo_X'
+                elif d == 'dolgo_1':
+                    d = 'dolgo_TONE'
+                elif d == 'dolgo_0':
+                    d = 'dolgo_ERROR'
+
+            if j in swaps:
+                tmp += td_swap.format(char, d)
+            elif local[j] != '*':
+                tmp += td_unaligned.format(char, d)
+            else:
+                tmp += td_residue.format(char, d)
+        out += tr.format(tmp, unique, taxa[taxon], seqs[sequence])
+
     html = html.format(
             table = out,
             dataset = dataset,
@@ -450,9 +604,11 @@ def msa2html(
             sequence = seq_id,
             shorttitle = shorttitle,
             width=len(msa['alignment'][0]),
-            table_width='{0}'.format(len(msa['alignment'][0])* 50 + 15 * taxl),
+            table_width='{0}'.format(len(msa['alignment'][0])* 50 + 8 * taxl),
             taxa = len(msa['alignment']),
-            uniseqs=len(set(msa['seqs']))
+            uniseqs=len(set(msa['seqs'])),
+            css = css,
+            js = js
             )
     
     if not filename:
@@ -461,6 +617,12 @@ def msa2html(
     if not filename.endswith('.html'):
         filename = filename+'.html'
 
+    if keywords['compact']:
+        html = html.replace('\n',' ')
+        html = re.sub(r'\s+',r' ',html)
+        html = html.replace('> ','>')
+        html = html.replace(' >','>')
+
     # check, whether the outfile already exists
     outf = codecs.open(filename,'w','utf-8')
     outf.write(html)
@@ -598,7 +760,7 @@ def msa2tex(
         body += r'\ttfamily '+taxon.replace('_',r'\_')
         for j,char in enumerate(msa['alignment'][i]):
             if char != '-':
-                cls = rcParams['dolgo'][char]
+                cls = token2class(char,rcParams['dolgo']) 
             elif char == '-':
                 cls = 'X'
             if char == '_':
@@ -639,3 +801,193 @@ def msa2tex(
     out.write(tex)
     out.close()
     if rcParams['verbose']: print(rcParams['M_file_written'].format(filename+'.tex'))
+
+
+def tokens2html(
+        string,
+        swaps = [],
+        tax_len = None,
+        path = '',
+        template = ''
+        ):
+    """
+    Function converts an (aligned) string into colored html-format.
+
+    """
+    # set the tr-line
+    tr = '<tr class="msa">\n{0}\n</tr>'
+
+    # get the percentage scaling factor
+    perc = int(80 / len(string) + 0.5)
+
+    # get vals for residue and swaps
+    td_residue = '<td class="residue" width="50" align="center" bgcolor="{1}">'+\
+            '<font color="{2}">{0}</font></td>\n'
+    td_swap = '<td class="residue swap" style="border:solid 3px black" width="50"'+\
+            'align="center" bgcolor="{1}"><font color="{2}">{0}</font></td>\n'
+
+    # start with filling the taxon
+    out = '<table>'
+    
+    # go on with the colors
+    for i,char in enumerate(string):
+        try:
+            c = rcParams['_color'][char]
+            fg = '#000000'
+        except:
+            try:
+                c = rcParams['_color'][char[0]]
+                fg = '#000000'
+            except:
+                input("Unknown character '"+char+"', press ANY key to continue. " )
+                c = '#ffffff'
+                fg = '#eb3410'
+
+        if i in swaps:
+            out += td_swap.format(char,c,fg)
+        else:
+            out += td_residue.format(char,c,fg)
+
+    return out+'</table>'
+
+
+def psa2html(filename, **keywords):
+    """
+    Function converts a PSA-file into colored html-format.
+    """
+
+    kw = dict(
+            template = False,
+            css = False,
+            comment = '#',
+            filename = filename.replace('.psa','.html'),
+            compact = True
+            )
+    kw.update(keywords)
+    
+    path = os.path.join(rcParams['_path'],'data','templates')
+    
+    if not kw['template']:
+        template = codecs.open(os.path.join(path, 'psa.html'), 'r',
+                'utf-8').read()
+    else:
+        template = codecs.open(kw['template'], 'r', 'utf-8').read()
+
+    if not kw['css']:
+        css = codecs.open(os.path.join(path, 'psa.css'), 'r',
+                'utf-8').read()
+    else:
+        css = codecs.open(kw['css'], 'r', 'utf-8').read()
+
+    raw_data = codecs.open(filename, 'r', 'utf-8')
+    data = []
+    for line in raw_data:
+        if not line.startswith(kw['comment']):
+            data += [line.strip()]
+    
+    seq_ids = []
+    pairs = []
+    taxa = []
+    alignments = []
+
+    del data[0]
+
+    i = 0
+    while i <= len(data) - 3:
+        try:
+            seq_ids.append(data[i])
+            
+            datA = data[i+1].split('\t')
+            datB = data[i+2].split('\t')
+            
+            taxonA = datA[0].strip('.')
+            taxonB = datB[0].strip('.')
+            almA = datA[1:]
+            almB = datB[1:]
+            
+            taxa.append((taxonA,taxonB))
+            pairs.append(
+                    (
+                        '.'.join([k for k in almA if k != '-']),
+                        '.'.join([k for k in almB if k != '-'])
+                        )
+                    )
+            alignments.append(
+                    (
+                        [str(a) for a in almA],
+                        [str(b) for b in almB],
+                        0)
+                    )
+            i += 4
+        except:
+            print("[!] Line {0} of the data is probablyb miscoded.".format(
+                i+1
+                ))
+            i += 1
+
+    
+    def get_classes(alm):
+
+        classes = []
+        residue = '<div class="residue {1}">{0}</div>'
+        for j,char in enumerate(alm):
+            if char == '-':
+                d = 'dolgo_GAP'
+            else:
+                d = 'dolgo_'+token2class(char, rcParams['dolgo'])
+
+                # bad check for three classes named differently
+                if d == 'dolgo__':
+                    d = 'dolgo_X'
+                elif d == 'dolgo_1':
+                    d = 'dolgo_TONE'
+                elif d == 'dolgo_0':
+                    d = 'dolgo_ERROR'
+            classes += [residue.format(char, d)]
+        return ''.join(classes)
+    
+    tr = '<tr class="psa">{0}</tr>'
+
+
+    out = '<table>\n' #codecs.open(kw['filename'], 'w', 'utf-8')
+    for i,(a,b,c) in enumerate(alignments):
+        
+        clsA = get_classes(a)
+        clsB = get_classes(b)
+
+        ids = int(100 * pid(a,b)+0.5)
+
+        out += '<tr class="head">'
+        out += '<td colspan=2 class="head"><b>Alignment {0}:</b> <i>{1}</i>, PID: {2}</td></tr>'.format(
+                i+1,
+                seq_ids[i],
+                ids
+                )
+        out += '<tr class="psa">'
+        out += '<td class="taxon">{0}</td>'.format(taxa[i][0])
+        out += '<td class="psa">{0}</td>'.format(clsA)
+        out += '</tr>'
+        out += '<tr class="psa">'
+        out += '<td class="taxon">{0}</td>'.format(taxa[i][1])
+        out += '<td class="psa">{0}</td>'.format(clsB)
+        out += '</tr>'
+        out += '<tr><td colspan=2></td></tr>'
+    
+    out += '</table>'
+    
+    html = template.format(
+            alignments = out,
+            css = css
+            )
+
+    if kw['compact']:
+        html = html.replace('\n',' ')
+        html = re.sub(r'\s+',r' ',html)
+        html = html.replace('> ','>')
+        html = html.replace(' >','>')
+
+    with codecs.open(kw['filename'], 'w', 'utf-8') as f:
+        f.write(html)
+    
+        
+       
