@@ -87,13 +87,14 @@ class Tokenizer(object):
     def __init__(self, orthography_profile=None):
         # check various places for orthography profiles
         ortho_path = ""
+
         if not orthography_profile == None:
             # strip possible file extension
             if orthography_profile.endswith('.prf'):
                 orthography_profile = orthography_profile[:-4]
 
             # get path and filename of orthography profile
-            if os.path.exists(orthography_profile+".prf"):
+            if os.path.exists(orthography_profile+".prf") or os.path.exists(orthography_profile+".rules"):
                 ortho_path = orthography_profile
             else:
                 ortho_path = os.path.join(
@@ -124,6 +125,7 @@ class Tokenizer(object):
 
         else:
             self.orthography_profile = None
+            self.column_labels = None
 
         # orthography profile rules and replacements
         if os.path.isfile(ortho_path+".rules"):
@@ -137,7 +139,8 @@ class Tokenizer(object):
         if rcParams['debug']: 
             print("[i] Orthography profile: ", self.orthography_profile)
             print("[i] Orthography rules: ", self.orthography_profile_rules)
-
+            print("[i] Columns labels: ", self.column_labels)
+            print()
     
     def _init_profile(self, f):
         """
@@ -352,6 +355,9 @@ class Tokenizer(object):
             Result of the transformation.
 
         """
+        # column labels are normalized 
+        column = column.lower()
+
         # This method can't be called unless an orthography profile was specified.
         if self.orthography_profile == None:
             raise Exception("This method only works when an orthography profile is specified.")
@@ -406,6 +412,9 @@ class Tokenizer(object):
 
         """
 
+        # column labels are normalized 
+        column = column.lower() 
+
         if self.orthography_profile and self.orthography_profile_rules:
             return self.rules(self.transform(string, column))
 
